from netgen.geom2d import SplineGeometry
import netgen.meshing as nm
import ngsolve as ng

geo = SplineGeometry()
geo.AddCircle([0, 0], 1)
mesh = ng.Mesh(geo.GenerateMesh(maxh=.2))

controlverts = [v for v in mesh.vertices if len(v.elements) == 0]
pts = [v.point for v in controlverts]

print("controlverts", controlverts)
print("pts", pts)

# Output is:

# Generate Mesh from spline geometry
# controlverts [V1, V3, V5, V7]
# pts [(1.0, -1.0), (1.0, 1.0), (-1.0, 1.0), (-1.0, -1.0)]

fes = ng.H1(mesh, order=1)
gfu = ng.GridFunction(fes)
gfu.vec[:] = 0
for v in mesh.vertices:
    mip = mesh(*v.point)
    value = gfu(mip)
    print("value", value)

# Output is:

# value 0.0
# WARNING: MeshPoint not in mesh, can't convert to BaseMappedIntegrationPoint!
# ---------------------------------------------------------------------------
# TypeError                                 Traceback (most recent call last)
# ~/ngstents/purepy/example.py in <module>
#      23 for v in mesh.vertices:
#      24     mip = mesh(*v.point)
# ---> 25     value = gfu(mip)
#      26     print("value", value)
#      27 

# TypeError: __call__(): incompatible function arguments. The following argument types are supported:
#     1. (self: ngsolve.fem.CoefficientFunction, mip: ngsolve.fem.BaseMappedIntegrationPoint) -> object
#     2. (self: ngsolve.fem.CoefficientFunction, x: float, y: Optional[float] = None, z: Optional[float] = None) -> ngfem::PointEvaluationFunctional
#     3. (self: ngsolve.fem.CoefficientFunction, arg0: numpy.ndarray[ngsolve.fem.MeshPoint]) -> numpy.ndarray

# Invoked with: <ngsolve.comp.GridFunctionD object at 0x7f23120f0db0>, <ngsolve.fem.MeshPoint object at 0x7f2354faf5b0>

